"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const shiki = __importStar(require("shiki"));
const shiki_languages_1 = require("shiki-languages");
const unist_util_visit_1 = __importDefault(require("unist-util-visit"));
const hast_util_to_string_1 = __importDefault(require("hast-util-to-string"));
const unist_builder_1 = __importDefault(require("unist-builder"));
const fast_copy_1 = __importDefault(require("fast-copy"));
function tokensToHast(lines) {
    let tree = [];
    for (const line of lines) {
        if (line.length === 0) {
            tree.push(unist_builder_1.default("text", "\n"));
        }
        else {
            for (const token of line) {
                tree.push(unist_builder_1.default("element", {
                    tagName: "span",
                    properties: {
                        style: `color: ${token.color};`,
                    },
                }, [unist_builder_1.default("text", token.content)]));
            }
            tree.push(unist_builder_1.default("text", "\n"));
        }
    }
    // Remove the last \n
    tree.pop();
    return tree;
}
function addStyle(node, style) {
    const props = (node.properties || {});
    props.style = props.style ? props.style + ";" + style : style;
    node.properties = props;
}
function addClass(node, className) {
    const props = (node.properties || {});
    props.className = props.className
        ? [...props.className, className]
        : [className];
    node.properties = props;
}
function codeLanguage(node) {
    const props = (node.properties || {});
    const className = props.className || [];
    let value;
    for (const element of className) {
        value = element;
        if (value.slice(0, 9) === "language-") {
            return value.slice(9);
        }
    }
    return null;
}
function highlightBlock(highlighter, node, options) {
    if (options.useBackground) {
        addStyle(node, "background: " + highlighter.getBackgroundColor());
    }
    const lang = codeLanguage(node);
    if (!lang) {
        // Unknown language, fall back to a foreground colour
        addStyle(node, "color: " + highlighter.getForegroundColor());
        return;
    }
    const tokens = highlighter.codeToThemedTokens(hast_util_to_string_1.default(node), lang);
    const tree = tokensToHast(tokens);
    node.children = tree;
}
async function getTheme(theme) {
    return typeof theme === "string"
        ? shiki.BUNDLED_THEMES.includes(theme)
            ? theme
            : shiki.loadTheme(theme)
        : theme;
}
async function getHighlighter(theme, langs = []) {
    const loadedTheme = await getTheme(theme);
    return shiki.getHighlighter({
        theme: loadedTheme,
        langs: [...shiki_languages_1.BUNDLED_LANGUAGES, ...langs],
    });
}
let lightHighlighterPromise;
let darkHighlighterPromise;
function attacher(options = {}) {
    const { theme = "github-light", darkTheme, useBackground = true, langs = [], } = options;
    lightHighlighterPromise || (lightHighlighterPromise = getHighlighter(theme, langs));
    if (darkTheme) {
        darkHighlighterPromise || (darkHighlighterPromise = getHighlighter(darkTheme, langs));
    }
    return async function transformer(tree) {
        const lightHighlighter = await lightHighlighterPromise;
        const darkHighlighter = await darkHighlighterPromise;
        unist_util_visit_1.default(tree, "element", (node, index, parent) => {
            if (!parent ||
                parent.tagName !== "pre" ||
                node.tagName !== "code" ||
                node.dark) {
                return;
            }
            highlightBlock(lightHighlighter, node, { useBackground });
            addClass(node, "syntax-light");
            const darkNode = fast_copy_1.default(node);
            darkNode.dark = true;
            addClass(darkNode, "syntax-dark");
            highlightBlock(darkHighlighter, darkNode, { useBackground });
            parent.children.splice(index + 1, 0, darkNode);
        });
    };
}
exports.default = attacher;
